/* ISC license. */

/* Generated by: ./configure --prefix=/usr --sysconfdir=/etc --disable-allstatic --disable-static --enable-pkgconfig --enable-shared --with-pkgconfig */

#ifndef S6_DNS_CONFIG_H
#define S6_DNS_CONFIG_H

#define S6_DNS_VERSION "2.4.1.3"
#define S6_DNS_BINPREFIX ""
#define S6_DNS_EXTBINPREFIX ""
#define S6_DNS_EXTLIBEXECPREFIX "/usr/libexec/"
#define S6_DNS_LIBEXECPREFIX "/usr/libexec/"
#define S6_DNS_SYSCONFPREFIX "/etc/"

#endif
